<?php
### Lynx Template
### Copyright (c) 2011 Jonathan Coulet - j.coulet@gmail.com
### http://lynx-template.phpixel.fr/
### Licensed under the GPL license.
###
### Lynx Template est basé sur le moteur de template Simple TPL (Copyright (c) 2009 Jonathan Coulet - j.coulet@gmail.com)
###
### TODO
### - méthode assign()
### - supression des LOOP non parssés
### - réécriture méthode parse()
###
### V 1.1
### - réécriture des méthode loop(), assignVar(), remove(), parseVar() et assignVars()
### - optimisation de la méthode parse
### - ajout méthode copyLoop()
### - ajout du helper magique
### - ajout du temps d'execution de la méthode parse() dans l'affichage de debug()
###
### V 1.0
### - suppressions méthodes compression(), retire() et de la propriété $param
### - ajout des propriétés $buffer et $dir
### - renommages des propriétés, méthodes et helpers variables
### - modification de la syntaxe template
### - ajout méthodes debug(), display(), removeBlock()

class lynxTemplate{
	private $dir;
	private $buffer;
	private $aVar;
	private $aLoop;
	private $aOpt;
	private $parseTime;
	
	public function __construct($file, $dir = 'templates/'){
		$this->buffer = file_get_contents($dir.$file);
		$this->aVar = array();
		$this->aLoop = array();
		$this->aOpt = array();
		$this->dir = $dir;
	}
	
	###
	### assignVars()
	### alimente le tableau $aVar
	###
	
	public function assignVars($vars){
		foreach($vars as $name=>$value){
			$this->assignVar($name, $value);
		}
	}
	
	###
	### assignVar()
	### alimente le tableau $aVar
	###
	
	public function assignVar($name, $value){
		$this->aVar[$name] = $value;
	}
	
	###
	### loop()
	### alimente le tableau $aLoop
	###
	
	public function loop($name, $vars){
		if(!isset($this->aLoop[$name])) $this->aLoop[$name] = array();
		array_push($this->aLoop[$name], $vars);
	}
	
	###
	### remove()
	### alimente le tableau $aOpt
	###
	
	public function remove($name){
		$this->aOpt[] = $name;
	}
	
	###
	### copyLoop()
	### duplique un bloc répétitif
	###
	
	public function copyLoop($source, $target){
		$this->aLoop[$target] = $this->aLoop[$source];
	}
	
	### 
	### debug()
	###
	
	public function debug(){
		echo '<b>parseTime : </b>'.$this->parseTime.'<br />';
		echo '<b>$aVar :</b><br />';
		print_r($this->aVar);
		echo '<br />';
		echo '<b>$aLoop :</b><br />';
		print_r($this->aLoop);
		echo '<br />';
		echo '<b>$aOpt :</b><br />';
		print_r($this->aOpt);
		echo '<br />';
	}
	
	### 
	### display()
	###
	
	public function display(){
		echo $this->parse();
	}
	
	/*
	** Parse
	*/
	public function parse(){
		$timeStart = microtime(true);
		// Includes
		if(preg_match_all('#<!--INC:(.*)-->#Usi', $this->buffer, $matches)){
			foreach($matches[1] as $k=>$fichier){
				if(file_exists($this->dir.$fichier)){
					$contenu = file_get_contents($this->dir.$fichier);
					$this->buffer = str_replace($matches[0][$k], $contenu, $this->buffer);
				}
			}
		}
		// Suppression des blocks optionnels
		foreach($this->aOpt as $nom){
			$this->removeBlock('opt', $nom);
		}
		// Parssage des variables
		foreach($this->aVar as $nom=>$contenu){
			$this->buffer = $this->parseVar($nom, $contenu, $this->buffer);
		}
		// Parssage des boucles
		foreach($this->aLoop as $nomBoucle=>$a_variables){
			$lignes = '';
			$ligneModele = '';
			// Récupération de la ligne a parser
			preg_match('#<!--LOOP:'.$nomBoucle.'-->(.*)<!--ENDLOOP:'.$nomBoucle.'-->#Usi', $this->buffer, $matches);
			$ligneModele = $matches[0];
			// Parssage des lignes
			foreach($a_variables as $variables){
				$ligneTemp = $ligneModele;
				foreach($variables as $nomVar=>$contenuVar){
					$ligneTemp = $this->parseVar($nomBoucle.'.'.$nomVar, $contenuVar, $ligneTemp);
				}
				$lignes .= $ligneTemp;
			}
			$this->buffer = preg_replace('#<!--LOOP:'.$nomBoucle.'-->(.*)<!--ENDLOOP:'.$nomBoucle.'-->#Usi', $lignes, $this->buffer);
			$this->removeBlock('loop', $nomBoucle, false);
		}
		$this->parseTime = microtime(true)-$timeStart;
		return $this->buffer;
	}
	
	###
	### removeBlock()
	### supprime un bloc et son contenu
	### si $removeContent est false le contenu ne sera pas supprimé
	###
	
	private function removeBlock($type, $name, $removeContent = true){
		if($removeContent) $mask = '#<!--'.mb_strtoupper($type).':'.$name.'-->(.*)<!--END'.mb_strtoupper($type).':'.$name.'-->#Usi';
		else $mask = '#<!--(END'.mb_strtoupper($type).'|'.mb_strtoupper($type).'):'.$name.'-->#Usi';
		$this->buffer = preg_replace($mask, '', $this->buffer);
	}
	
	###
	### parseVar()
	### parse les variables d'une chaine
	### applique un helper sur les variables d'une chaine
	###
	
	private function parseVar($name, $value, $content){
		$tag = '{'.$name.'}';
		if(preg_match('#{'.$name.':(.*)}#Usi', $content, $matches)){
			$tag = $matches[0];
			switch($matches[1]){
				case 'ucw':
					$value = ucwords($value);
					break;
				case 'up':
					$value = mb_strtoupper($value);
					break;
				case 'low':
					$value = mb_strtolower($value);
					break;
				case 'ucf':
					$value = ucfirst($value);
					break;
				default:
					if(is_numeric($matches[1])) $value = mb_strcut($value, 0, $matches[1]).'...';
			}
		}
		return str_replace($tag, $value, $content);
	}
}
?>